#!/bin/bash

if ! [ -d "$OSGEO4W_REP" ]; then
	echo OSGEO4W_REP not set
	exit 1
fi

D="$(cd $(dirname "$0") && pwd)"

rm -f $D/package.log.9
for i in $(seq 8 -1 1); do
	[ -f $D/package.log.$i ] && mv $D/package.log.$i $D/package.log.$(( i + 1 ))
done
[ -f $D/package.log ] && mv $D/package.log $D/package.log.1

exec > >(tee package.log) 2>&1

if ! [ -f qt-unified-windows-x86-online.exe ]; then
	wget -c http://download.qt.io/official_releases/online_installers/qt-unified-windows-x86-online.exe
	chmod a+rx qt-unified-windows-x86-online.exe
fi

V=5.11.2
B=1
QV=${V//./}
MAINTAINER=JuergenFischer

export V QV B

if ! [ -d qt5 ]; then
	cat <<EOF >/tmp/qt.script
// see http://doc.qt.io/qtinstallerframework/
// see http://doc.qt.io/qtinstallerframework/noninteractive.html
// see http://doc.qt.io/qtinstallerframework/scripting-qmlmodule.html
function Controller() {
    console.log("Initializing the Controller...");
    installer.autoRejectMessageBoxes();
    installer.installationFinished.connect(function() {
        console.log("On installationFinished...");
        gui.clickButton(buttons.NextButton);
    });
}
Controller.prototype.WelcomePageCallback = function() {
    console.log("On WelcomePageCallback...");
    // NB we must add a delay because the next button is not immediately enabled.
    gui.clickButton(buttons.NextButton, 2000);
};
Controller.prototype.CredentialsPageCallback = function() {
    console.log("On CredentialsPageCallback...");
    gui.clickButton(buttons.NextButton);
};
Controller.prototype.IntroductionPageCallback = function() {
    console.log("On IntroductionPageCallback...");
    gui.clickButton(buttons.NextButton);
    console.log("Retrieving meta information from remote repository...");
};
Controller.prototype.TargetDirectoryPageCallback = function() {
    console.log("On TargetDirectoryPageCallback...");
    // NB the target directory cannot contain spaces.
    gui.currentPageWidget().TargetDirectoryLineEdit.setText("$(cygpath -m $PWD/qt5)");
    gui.clickButton(buttons.NextButton);
}
Controller.prototype.ComponentSelectionPageCallback = function() {
    console.log("On ComponentSelectionPageCallback...");
    var widget = gui.currentPageWidget();
    widget.deselectAll();
    widget.selectComponent("qt.qt5.$QV.debug_info");
    widget.selectComponent("qt.qt5.$QV.debug_info.win32_msvc2015");
    widget.selectComponent("qt.qt5.$QV.debug_info.win64_msvc2015_64");
    widget.selectComponent("qt.qt5.$QV.doc");
    widget.selectComponent("qt.qt5.$QV.doc.qtcharts");
    widget.selectComponent("qt.qt5.$QV.doc.qtdatavis3d");
    widget.selectComponent("qt.qt5.$QV.doc.qtnetworkauth");
    widget.selectComponent("qt.qt5.$QV.doc.qtscript");
    widget.selectComponent("qt.qt5.$QV.doc.qtwebengine");
    widget.selectComponent("qt.qt5.$QV.qtcharts");
    widget.selectComponent("qt.qt5.$QV.qtcharts.win32_msvc2015");
    widget.selectComponent("qt.qt5.$QV.qtcharts.win64_msvc2015_64");
    widget.selectComponent("qt.qt5.$QV.qtdatavis3d");
    widget.selectComponent("qt.qt5.$QV.qtdatavis3d.win32_msvc2015");
    widget.selectComponent("qt.qt5.$QV.qtdatavis3d.win64_msvc2015_64");
    widget.selectComponent("qt.qt5.$QV.qtnetworkauth");
    widget.selectComponent("qt.qt5.$QV.qtnetworkauth.win32_msvc2015");
    widget.selectComponent("qt.qt5.$QV.qtnetworkauth.win64_msvc2015_64");
    widget.selectComponent("qt.qt5.$QV.qtscript");
    widget.selectComponent("qt.qt5.$QV.qtscript.win32_msvc2015");
    widget.selectComponent("qt.qt5.$QV.qtscript.win64_msvc2015_64");
    widget.selectComponent("qt.qt5.$QV.qtwebengine");
    widget.selectComponent("qt.qt5.$QV.win32_msvc2015");
    widget.selectComponent("qt.qt5.$QV.win64_msvc2015_64");
    gui.clickButton(buttons.NextButton);
};
Controller.prototype.LicenseAgreementPageCallback = function() {
    console.log("On LicenseAgreementPageCallback...");
    gui.currentPageWidget().AcceptLicenseRadioButton.checked = true;
    gui.clickButton(buttons.NextButton);
};
Controller.prototype.StartMenuDirectoryPageCallback = function() {
    console.log("On StartMenuDirectoryPageCallback...");
    gui.currentPageWidget().StartMenuPathLineEdit.setText("Qt5");
    gui.clickButton(buttons.NextButton);
};
Controller.prototype.ReadyForInstallationPageCallback = function() {
    console.log("On ReadyForInstallationPageCallback...");
    gui.clickButton(buttons.NextButton);
};
Controller.prototype.PerformInstallationPageCallback = function() {
    console.log("On PerformInstallationPageCallback...");
    gui.clickButton(buttons.NextButton);
};
Controller.prototype.FinishedPageCallback = function() {
    console.log("On FinishedPageCallback...");
    gui.currentPageWidget().LaunchQtCreatorCheckBoxForm.launchQtCreatorCheckBox.checked = false;
    gui.clickButton(buttons.FinishButton);
};
EOF

	./qt-unified-windows-x86-online.exe --verbose --script $(cygpath -m /tmp/qt.script)
fi

set -e

for A in x86 x86_64; do
	case $A in
	x86)
		export QTB=msvc2015
		;;
	x86_64)
		export QTB=msvc2015_64
		;;
	*)
		false
		;;
	esac

	rm -f /tmp/debuglibs-$A /tmp/debuglibs-pdb-$A

	cd $D/qt5/$V/$QTB

	find . -name "*.pdb" | while read f; do
		f=${f#./}
		f=${f%.pdb}
		d=${f%d}

		if [ "$f" != "$d" -a -f "${f/bin\//lib\/}.lib" -a -f "${d/bin\//lib\/}.lib" ]; then
			echo $f.pdb >>/tmp/debuglibs-pdb-$A
			for i in $f.dll ${f/bin\//lib\/}.prl ${f/bin\//lib\/}.lib; do
				[ -f "$i" ] && echo $i >>/tmp/debuglibs-$A
			done
		fi
	done

	QT5REP=$OSGEO4W_REP/$A/release/qt5

	for i in devel doc qml tools libs libs-pdb libs-debug libs-debug-pdb; do
		mkdir -p $QT5REP/qt5-$i
	done

	cat <<EOF >$QT5REP/qt5-libs-debug/setup.hint
sdesc: "Qt5 runtime libraries (debug version)"
ldesc: "Qt5 runtime libraries (debug version)"
maintainer: $MAINTAINER
category: Libs
requires: shell zlib
external-source: qt5-libs
EOF
	tar -cjf $QT5REP/qt5-libs-debug/qt5-libs-debug-$V-$B.tar.bz2 \
		-T /tmp/debuglibs-$A \
		--xform s,^,apps/qt5/,
	tar -tjf $QT5REP/qt5-libs-debug/qt5-libs-debug-$V-$B.tar.bz2 >$QT5REP/qt5-libs-debug/qt5-libs-debug-$V-$B.manifest

	cat <<EOF >$QT5REP/qt5-libs-debug-pdb/setup.hint
sdesc: "Qt5 runtime libraries (debug versions; PDBs)"
ldesc: "Qt5 runtime libraries (debug versions; PDBs)"
maintainer: $MAINTAINER
category: Libs
requires: qt5-libs-debug
external-source: qt5-libs
EOF
	tar -cjf $QT5REP/qt5-libs-debug-pdb/qt5-libs-debug-pdb-$V-$B.tar.bz2 \
		-T /tmp/debuglibs-pdb-$A \
		--xform s,^,apps/qt5/,
	tar -tjf $QT5REP/qt5-libs-debug-pdb/qt5-libs-debug-pdb-$V-$B.tar.bz2 >$QT5REP/qt5-libs-debug-pdb/qt5-libs-debug-pdb-$V-$B.manifest

	cat <<EOF >$QT5REP/qt5-tools/setup.hint
sdesc: "Qt5 designer & linguist (Development)"
ldesc: "Qt5 designer & linguist (Development)"
maintainer: $MAINTAINER
category: Libs
requires: shell qt5-libs
external-source: qt5-libs
EOF
	tar -cjf $QT5REP/qt5-tools/qt5-tools-$V-$B.tar.bz2 \
		--xform s,^,apps/qt5/, \
		translations/linguist_*.qm \
		translations/designer_*.qm \
		bin/designer.exe \
		bin/linguist.exe
	tar -tjf $QT5REP/qt5-tools/qt5-tools-$V-$B.tar.bz2 >$QT5REP/qt5-tools/qt5-tools-$V-$B.manifest

	cat <<EOF >$QT5REP/qt5-devel/setup.hint
sdesc: "Qt5 headers and libraries (Development)"
ldesc: "Qt5 headers and libraries (Development)"
maintainer: $MAINTAINER
category: Libs
requires: qt5-libs
external-source: qt5-libs
EOF
	tar -cjf $QT5REP/qt5-devel/qt5-devel-$V-$B.tar.bz2 \
		--exclude-from /tmp/debuglibs-$A \
		--exclude "bin/linguist.exe" \
		--exclude "bin/designer.exe" \
		--exclude "*.pdb" \
		--xform s,^,apps/qt5/, \
		include \
		mkspecs \
		phrasebooks \
		lib \
		translations/assistant_*.qm \
		bin/*.exe
	tar -tjf $QT5REP/qt5-devel/qt5-devel-$V-$B.tar.bz2 >$QT5REP/qt5-devel/qt5-devel-$V-$B.manifest

	cat <<EOF >$QT5REP/qt5-doc/setup.hint
sdesc: "Qt5 documentation (Development)"
ldesc: "Qt5 documentation (Development)"
maintainer: $MAINTAINER
category: Libs
requires: 
external-source: qt5-libs
EOF
	tar -cjf $QT5REP/qt5-doc/qt5-doc-$V-$B.tar.bz2 \
		-C $D/qt5 \
		--xform s,Docs/Qt-$V,apps/qt5/doc, \
		Docs/Qt-$V
	tar -tjf $QT5REP/qt5-doc/qt5-doc-$V-$B.tar.bz2 >$QT5REP/qt5-doc/qt5-doc-$V-$B.manifest

	cat <<EOF >$QT5REP/qt5-qml/setup.hint
sdesc: "Qt5 QML"
ldesc: "Qt5 QML"
maintainer: $MAINTAINER
category: Libs
requires: qt5-libs
external-source: qt5-libs
EOF
	tar -cjf $QT5REP/qt5-qml/qt5-qml-$V-$B.tar.bz2 \
		--exclude "*.pdb" \
		--exclude-from /tmp/debuglibs-$A \
		--xform s,^,apps/qt5/, \
		qml
	tar -tjf $QT5REP/qt5-qml/qt5-qml-$V-$B.tar.bz2 >$QT5REP/qt5-qml/qt5-qml-$V-$B.manifest

	cat <<EOF >$D/qt5_env.bat
@echo off
path %OSGEO4W_ROOT%\\apps\\qt5\\bin;%PATH%

set QT_PLUGIN_PATH=%OSGEO4W_ROOT%\\apps\\Qt5\\plugins

set O4W_QT_PREFIX=%OSGEO4W_ROOT:\\=/%/apps/Qt5
set O4W_QT_BINARIES=%OSGEO4W_ROOT:\\=/%/apps/Qt5/bin
set O4W_QT_PLUGINS=%OSGEO4W_ROOT:\\=/%/apps/Qt5/plugins
set O4W_QT_LIBRARIES=%OSGEO4W_ROOT:\\=/%/apps/Qt5/lib
set O4W_QT_TRANSLATIONS=%OSGEO4W_ROOT:\\=/%/apps/Qt5/translations
set O4W_QT_HEADERS=%OSGEO4W_ROOT:\\=/%/apps/Qt5/include
set O4W_QT_DOC=%OSGEO4W_ROOT:\\=/%/apps/Qt5/doc
EOF

	cat <<EOF >$D/qt5.conf
[Paths]
Prefix=\$(O4W_QT_PREFIX)
Binaries=\$(O4W_QT_BINARIES)
Plugins=\$(O4W_QT_PLUGINS)
Libraries=\$(O4W_QT_LIBRARIES)
Translations=\$(O4W_QT_TRANSLATIONS)
Headers=\$(O4W_QT_HEADERS)
Documentation=\$(O4W_QT_DOC)
EOF

	cat <<EOF >$D/qt5-libs-postinstall.bat
copy "%OSGEO4W_ROOT%"\\bin\\qt5.conf "%OSGEO4W_ROOT%"\\bin\\qt.conf
EOF

	cat <<EOF >$QT5REP/qt5-libs/setup.hint
sdesc: "Qt5 runtime libraries"
ldesc: "Qt5 runtime libraries"
maintainer: $MAINTAINER
category: Libs
requires: zlib openssl sqlite3 msvcrt2015
EOF
	tar -cjf $QT5REP/qt5-libs/qt5-libs-$V-$B.tar.bz2 \
		--absolute-names \
		--exclude-from /tmp/debuglibs-$A \
		--exclude-from /tmp/debuglibs-pdb-$A \
		--exclude "./include" \
		--exclude "./mkspecs" \
		--exclude "./phrasebooks" \
		--exclude "./lib" \
		--exclude "./qml" \
		--exclude "./doc" \
		--exclude "./bin/*.exe" \
		--exclude "./translations/assistant_*.qm" \
		--exclude "./translations/linguist_*.qm" \
		--exclude "./translations/designer_*.qm" \
		--exclude "*.pdb" \
		--xform s,$D/qt5_env.bat,bin/qt5_env.bat, \
		--xform s,$D/qt5-libs-postinstall.bat,etc/postinstall/qt5-libs.bat, \
		--xform s,$D/qt5.conf,bin/qt5.conf, \
		--xform s,\\./,apps/qt5/, \
		$D/qt5_env.bat \
		$D/qt5.conf \
		$D/qt5-libs-postinstall.bat \
		.
	tar -tjf $QT5REP/qt5-libs/qt5-libs-$V-$B.tar.bz2 >$QT5REP/qt5-libs/qt5-libs-$V-$B.manifest

	tar -cjf $QT5REP/qt5-libs/qt5-libs-$V-$B-src.tar.bz2 \
		-C $D/.. \
		osgeo4w/package.sh

	cat <<EOF >$QT5REP/qt5-libs-pdb/setup.hint
sdesc: "Qt5 runtime libraries (PDBs)"
ldesc: "Qt5 runtime libraries (PDBs)"
maintainer: $MAINTAINER
category: Libs
requires: qt5-libs
external-source: qt5-libs
EOF

	find bin lib qml plugins -name "*.pdb" |
		tar -cjf $QT5REP/qt5-libs-pdb/qt5-libs-pdb-$V-$B.tar.bz2 \
			--exclude-from /tmp/debuglibs-$A \
			--exclude-from /tmp/debuglibs-pdb-$A \
			--xform s,^,apps/qt5/, \
			-T -
	tar -tjf $QT5REP/qt5-libs-pdb/qt5-libs-pdb-$V-$B.tar.bz2 >$QT5REP/qt5-libs-pdb/qt5-libs-pdb-$V-$B.manifest

	(
		cd $D
		find qt5/$V/$QTB -type f | fgrep -v "qt5/$V/$QTB/doc/global/"
		find qt5 -type f | fgrep -v "qt5/$V/" | fgrep -v "qt5/Tools/QtCreator" | fgrep -v "qt5/Examples/"
	) |
	sed -e "s#^qt5/Docs/Qt-5.11.2/#apps/qt5/doc/#"  -e "s#^qt5/$V/$QTB#apps/qt5#" |
	sort >/tmp/srcfiles-$A

	sort \
		$QT5REP/qt5-libs-debug/qt5-libs-debug-$V-$B.manifest \
		$QT5REP/qt5-libs-debug-pdb/qt5-libs-debug-pdb-$V-$B.manifest \
		$QT5REP/qt5-tools/qt5-tools-$V-$B.manifest \
		$QT5REP/qt5-devel/qt5-devel-$V-$B.manifest \
		$QT5REP/qt5-doc/qt5-doc-$V-$B.manifest \
		$QT5REP/qt5-qml/qt5-qml-$V-$B.manifest \
		$QT5REP/qt5-libs/qt5-libs-$V-$B.manifest \
		$QT5REP/qt5-libs-pdb/qt5-libs-pdb-$V-$B.manifest |
		grep -v "/$" >/tmp/dstfiles-$A

	uniq -cd /tmp/dstfiles-$A >/tmp/dupes-$A

	diff -u /tmp/srcfiles-$A /tmp/dstfiles-$A || true
	wc -l /tmp/dupes-$A
done

exit ### END
